@@ -3,7 +3,7 @@ use crate::utils::callers::CurrentCaller;
 use crate::{debug, error, info, warn};
 use aes_gcm::aead::generic_array::{typenum, GenericArray};
 use anyhow::{anyhow, Context, Result};
-#[cfg(test)]
+#[cfg(all(test, feature = "yubikey"))]
 use mockall::automock;
 use serde::{de, Deserialize, Serialize};
 use std::cell::RefCell;
@@ -542,7 +542,7 @@ impl Database {
         id_seckey: crypto_box::SecretKey,
         group: crate::keepassxc::Group,
     ) -> Self {
-        let id_seckey_b64 = base64::encode(id_seckey.to_bytes());
+        let id_seckey_b64 = base64::encode(id_seckey.as_bytes());
         let id_pubkey = id_seckey.public_key();
         let id_pubkey_b64 = base64::encode(id_pubkey.as_bytes());
         Self {
@@ -736,10 +736,10 @@ trait YubiKeyTrait {
     ) -> Result<Vec<u8>, YubicoError>;
 }
 
-#[cfg(test)]
+#[cfg(all(test, feature = "yubikey"))]
 impl MockYubiKeyTrait {
     fn new_mock() -> Self {
-        use hmac::{Mac, NewMac};
+        use hmac::Mac;
 
         let mut mock_yubikey = Self::new();
         mock_yubikey
@@ -833,14 +833,19 @@ mod tests {
     use super::*;
     use crate::keepassxc::Group;
     use crate::utils::generate_secret_key;
+    #[cfg(feature = "yubikey")]
     use hmac::Hmac;
+    #[cfg(feature = "yubikey")]
     use sha1::Sha1;
     #[cfg(unix)]
     use std::os::unix::fs::PermissionsExt;
 
+    #[cfg(feature = "yubikey")]
     pub type HmacSha1 = Hmac<Sha1>;
 
+    #[cfg(feature = "yubikey")]
     pub static TEST_YUBIKEY_SERIAL: u32 = 1234567;
+    #[cfg(feature = "yubikey")]
     pub static TEST_YUBIKEY_HMAC_SHA1_SECRET: &'static str = "test_secret";
 
     #[test]
@@ -875,13 +880,14 @@ mod tests {
             assert_eq!(config.count_databases(), 1);
             let databases = config.get_databases().unwrap();
             assert_eq!(databases[0].id, database.id);
-            assert_eq!(databases[0].key, base64::encode(secret_key.to_bytes()));
+            assert_eq!(databases[0].key, base64::encode(secret_key.as_bytes()));
         }
 
         fs::remove_file(config_path).unwrap();
     }
 
     #[test]
+    #[cfg(feature = "yubikey")]
     fn test_01_config_read_write_challenge_response() {
         let config_path = {
             let mut temp = std::env::temp_dir();
@@ -921,7 +927,7 @@ mod tests {
             assert_eq!(config.count_databases(), 1);
             let databases = config.get_databases().unwrap();
             assert_eq!(databases[0].id, database.id);
-            assert_eq!(databases[0].key, base64::encode(secret_key.to_bytes()));
+            assert_eq!(databases[0].key, base64::encode(secret_key.as_bytes()));
             let decrypted = config.decrypt_databases().unwrap();
             assert_eq!(decrypted, 1);
             config.write_to(&config_path).unwrap();
@@ -935,6 +941,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(feature = "yubikey")]
     fn test_02_ignore_plaintext_callers_when_there_are_encrypted_ones() {
         let config_path = {
             let mut temp = std::env::temp_dir();
