@@ -14,15 +14,17 @@ use once_cell::unsync::OnceCell;
 use std::cell::RefCell;
 use std::env;
 use std::fmt;
-use std::io::{Read, Write};
+use std::io::{ErrorKind, Read, Write};
 #[cfg(unix)]
 use std::os::unix::net::UnixStream;
 use std::path::PathBuf;
 use std::rc::Rc;
 use std::str;
+use std::time::Duration;
 
 #[cfg(windows)]
 const NAMED_PIPE_CONNECT_TIMEOUT_MS: u32 = 100;
+const READ_TIMEOUT: Duration = Duration::new(0, 200 * 1_000_000);
 const KEEPASS_SOCKET_NAME: &str = "org.keepassxc.KeePassXC.BrowserServer";
 const KEEPASS_SOCKET_NAME_LEGACY: &str = "kpxc_server";
 const KEEPASS_SOCKET_ENVIRONMENT_VARIABLE: &str = "KEEPASSXC_BROWSER_SOCKET_PATH";
@@ -100,6 +102,21 @@ pub fn get_socket_path() -> Result<PathBuf> {
                     legacy_path.to_string_lossy()
                 );
             }
+            #[cfg(windows)]
+            if let Ok(username) = &std::env::var("USERNAME") {
+                let kpnatmsg_str = "keepassxc\\".to_owned() + username + "\\kpxc_server";
+                let kpnatmsg_path = get_socket_path_with_name(&kpnatmsg_str);
+                if let Ok(kpnatmsg_path) = kpnatmsg_path {
+                    if kpnatmsg_path.exists() {
+                        return Ok(kpnatmsg_path);
+                    }
+                    if PipeClient::connect_ms(&kpnatmsg_path, NAMED_PIPE_CONNECT_TIMEOUT_MS).is_ok()
+                    {
+                        return Ok(kpnatmsg_path);
+                    }
+                    info!("KeePassNatMsg socket path {} does not exist", kpnatmsg_str);
+                }
+            }
             let path = get_socket_path_with_name(KEEPASS_SOCKET_NAME)?;
             #[cfg(windows)]
             if PipeClient::connect_ms(&path, NAMED_PIPE_CONNECT_TIMEOUT_MS).is_ok() {
@@ -154,14 +171,16 @@ fn get_stream() -> Result<Rc<RefCell<UnixStream>>> {
     STREAM.with(|s| -> Result<_> {
         Ok(s.get_or_try_init(|| -> Result<_> {
             let path = get_socket_path()?;
-            Ok(Rc::new(RefCell::new(
-                UnixStream::connect(&path).with_context(|| {
-                    format!(
-                        "Failed to connect to Unix socket {}",
-                        path.to_string_lossy()
-                    )
-                })?,
-            )))
+            let stream = UnixStream::connect(&path).with_context(|| {
+                format!(
+                    "Failed to connect to Unix socket {}",
+                    path.to_string_lossy()
+                )
+            })?;
+            if let Err(e) = stream.set_read_timeout(Some(READ_TIMEOUT)) {
+                warn!("Failed to set read timeout: {}", e);
+            }
+            Ok(Rc::new(RefCell::new(stream)))
         })?
         .clone())
     })
@@ -173,11 +192,12 @@ fn get_stream() -> Result<Rc<RefCell<PipeClient>>> {
     STREAM.with(|s| -> Result<_> {
         Ok(s.get_or_try_init(|| -> Result<_> {
             let path = get_socket_path()?;
-            Ok(Rc::new(RefCell::new(
-                PipeClient::connect_ms(&path, NAMED_PIPE_CONNECT_TIMEOUT_MS).with_context(
-                    || format!("Failed to connect to named pipe {}", path.to_string_lossy()),
-                )?,
-            )))
+            let mut stream = PipeClient::connect_ms(&path, NAMED_PIPE_CONNECT_TIMEOUT_MS)
+                .with_context(|| {
+                    format!("Failed to connect to named pipe {}", path.to_string_lossy())
+                })?;
+            stream.set_read_timeout(Some(READ_TIMEOUT));
+            Ok(Rc::new(RefCell::new(stream)))
         })?
         .clone())
     })
@@ -244,10 +264,27 @@ impl MessagingUtilsInternalTrait for MessagingUtils {
         let stream_rc = get_stream()?;
         let mut stream = stream_rc.borrow_mut();
         let mut response = String::new();
-        const BUF_SIZE: usize = 128;
+        const BUF_SIZE: usize = 2048;
         let mut buf = [0u8; BUF_SIZE];
         loop {
-            let len = stream.read(&mut buf)?;
+            let len = match stream.read(&mut buf) {
+                Ok(len) if len <= BUF_SIZE => len,
+                Ok(len) => {
+                    warn!("Read returned {} > BUF_SIZE ({})", len, BUF_SIZE);
+                    BUF_SIZE
+                }
+                Err(e) if e.kind() == ErrorKind::TimedOut || e.kind() == ErrorKind::WouldBlock => {
+                    0usize
+                }
+                Err(e) => return Err(e.into()),
+            };
+            #[cfg(debug_assertions)]
+            {
+                debug!("Received {} bytes: {:?}", len, &buf[0..len]);
+                if let Ok(buf_str) = str::from_utf8(&buf[0..len]) {
+                    debug!("Received {} chars: {}", buf_str.len(), buf_str);
+                }
+            }
             response.push_str(str::from_utf8(&buf[0..len]).unwrap());
             if len < BUF_SIZE {
                 break;
